package DE_EPAGES::OAuthTokenClient::t::OAuthTokenIntegration;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient;
use DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient;
use DE_EPAGES::OAuthTokenClient::API::Object::OAuthAccessToken;
use DE_EPAGES::OAuthTokenClient::API::Object::OAuthConsumer;
use Test::More tests => 23;
use Test::Exception;

use strict;

use constant SHOP_ALIAS => 'TestShop';
use constant CONSUMER_KEY => 'perltest_consumer_key';

## no critic qw( ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->testPUT;
    $self->testPOST;
}

sub testPUT {

    my $ConsumerIn = DE_EPAGES::OAuthTokenClient::API::Object::OAuthConsumer->new(
        key => CONSUMER_KEY,
        name => 'Mr. Test',
        secret => 'perltest_consumer_secret',
    );
    my $ConsumerClient = DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient->create;
    $ConsumerClient->saveConsumer($ConsumerIn);
    my $ConsumerOut = $ConsumerClient->getConsumer($ConsumerIn->key);
    is $ConsumerOut->key, $ConsumerIn->key, 'saveConsumer.key';
    is $ConsumerOut->name, $ConsumerIn->name, 'saveConsumer.name';
    is $ConsumerOut->secret, $ConsumerIn->secret, 'saveConsumer.secret';

    my $TokenIn = DE_EPAGES::OAuthTokenClient::API::Object::OAuthAccessToken->new(
        token => 'perltest_token',
        secret => 'perltest_token_secret',
        context => 'sf',
        shopAlias => SHOP_ALIAS,
        consumerKey => CONSUMER_KEY,
    );
    my $TokenClient = DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient->create;
    $TokenClient->saveToken(SHOP_ALIAS, $TokenIn);
    my $TokenOut = $TokenClient->getToken(SHOP_ALIAS, $TokenIn->token);
    is $TokenOut->token, $TokenIn->token, 'saveToken.token';
    is $TokenOut->secret, $TokenIn->secret, 'saveToken.secret';
    is $TokenOut->context, $TokenIn->context, 'saveToken.context';
    is $TokenOut->shopAlias, $TokenIn->shopAlias, 'saveToken.shopAlias';
    is $TokenOut->consumerKey, $TokenIn->consumerKey, 'saveToken.consumerKey';

    $TokenClient->deleteToken(SHOP_ALIAS, $TokenIn->token);
}

sub testPOST {

    my $ConsumerClient = DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient->create;
    my $Consumer = $ConsumerClient->createConsumer('PerlConsumerName');
    is $Consumer->name, 'PerlConsumerName', 'createConsumer.name';
    ok length($Consumer->key) > 20, 'createConsumer.key';
    ok length($Consumer->secret) > 20, 'createConsumer.secret';

    my $TokenClient = DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient->create;
    my $Token = $TokenClient->createToken(SHOP_ALIAS, {
        consumerKey => $Consumer->key,
        roles => [ 'sf' ],
    });
    ok length($Token->consumerKey) > 20, 'createToken.consumerKey';
    ok length($Token->token) > 20, 'createToken.token';
    ok length($Token->secret) > 20, 'createToken.secret';
    is $Token->context, 'sf', 'createToken.context';
    is $Token->shopAlias, SHOP_ALIAS, 'createToken.shopAlias';

    my $Token1 = $TokenClient->getToken(SHOP_ALIAS, $Token->token);
    is $Token1->token, $Token->token, 'getToken.token';
    is $Token1->secret, $Token->secret, 'getToken.secret';

    my $Token2 = $TokenClient->createToken(SHOP_ALIAS, {
        consumerKey => $Consumer->key,
        roles => [ 'sf' ],
    });

    isnt $Token1->token, $Token2->token, 'new token';
    isnt $Token1->secret, $Token2->secret, 'new secret';

    my $aTokens = $TokenClient->getTokens(SHOP_ALIAS);
    is scalar @$aTokens, 2, 'getTokens';
    is $aTokens->[0]->{'consumerKey'}, $Consumer->key, 'getTokens[0].consumerKey';

    $TokenClient->deleteToken(SHOP_ALIAS, $Token->token);
    throws_ok sub { $TokenClient->getToken(SHOP_ALIAS, $Token->token) }, qr/Not Found/i, 'deleteToken';
}

sub cleanup {

    my $TokenClient = DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient->create;
    $TokenClient->deleteToken(SHOP_ALIAS, $_->{'token'}) foreach @{ $TokenClient->getTokens(SHOP_ALIAS) };

    my $ConsumerClient = DE_EPAGES::OAuthTokenClient::API::OAuth::ConsumerClient->create;
    eval { $ConsumerClient->deleteConsumer(CONSUMER_KEY); }; ## no critic qw(RequireExistsError);
}

__PACKAGE__->runScript;
