#========================================================================================
# package      DE_EPAGES::RemoteSearch::t::SearchReplicationToken
# state        private
#========================================================================================
package DE_EPAGES::RemoteSearch::t::SearchReplicationToken;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::RemoteSearch::API::Table::SearchReplicationToken qw (
    ExistsSearchReplicationToken
    GetAllPKeySearchReplicationTokens
    IncrementAndGetReplicationToken
    DeleteOldReplicationTokens
);
use Test::More;

use strict;

sub TestIncrementAndGetReplicationToken {
    my $self = shift;

    # this can fail, if we have other activity on the testsite... but should be fairly seldom
    my $dbi = GetCurrentDBHandle();

    $dbi->lockTables([
        { 'table' => 'searchreplicationtoken', 'mode' => 'WRITE'},
    ], sub {
        my $Token1 = IncrementAndGetReplicationToken();
        my $Token2 = IncrementAndGetReplicationToken();
        is( $Token1 + 1, $Token2, 'Increment and get token' );
        ok( ExistsSearchReplicationToken( $Token1 ), 'no automatic cleanup' );
        ok( ExistsSearchReplicationToken( $Token2 ), 'no automatic cleanup' );
    });
}

sub TestCleanupOldReplicationTokens {
    my $self = shift;

    # this can fail, if we have other activity on the testsite... but should be fairly seldom
    my $dbi = GetCurrentDBHandle();

    $dbi->lockTables([
        { 'table' => 'searchreplicationtoken', 'mode' => 'WRITE'},
    ], sub {
        my $LastToken;
        # two pages for DELETE ... IN
        for(1..110) {
            $LastToken = IncrementAndGetReplicationToken();
        }

        DeleteOldReplicationTokens();
        ok( ExistsSearchReplicationToken( $LastToken ), 'current token still alive' );
        is( scalar @{GetAllPKeySearchReplicationTokens()}, 1, 'automatic cleanup done' );
    });
}

sub test {
    TestIncrementAndGetReplicationToken();
    TestCleanupOldReplicationTokens();
    done_testing();
}

__PACKAGE__->runScript;
